==> actor-framework_0.13.2.orig.tar.gz: actor-framework-0.13.2/libcaf_core/src/try_match.cpp <==
/******************************************************************************
 *                       ____    _    _____                                   *
 *                      / ___|  / \  |  ___|    C++                           *
 *                     | |     / _ \ | |_       Actor                         *
 *                     | |___ / ___ \|  _|      Framework                     *
 *                      \____/_/   \_|_|                                      *
 *                                                                            *
 * Copyright (C) 2011 - 2015                                                  *
 * Dominik Charousset <dominik.charousset (at) haw-hamburg.de>                *
 *                                                                            *
 * Distributed under the terms and conditions of the BSD 3-Clause License or  *
 * (at your option) under the terms and conditions of the Boost Software      *
 * License 1.0. See accompanying files LICENSE and LICENSE_ALTERNATIVE.       *
 *                                                                            *
 * If you did not receive a copy of the license files, see                    *
 * http://opensource.org/licenses/BSD-3-Clause and                            *
 * http://www.boost.org/LICENSE_1_0.txt.                                      *
 ******************************************************************************/

#include "caf/detail/try_match.hpp"

namespace caf {
namespace detail {

using pattern_iterator = const meta_element*;

bool is_wildcard(const meta_element& me) {
  return me.typenr == 0 && me.type == nullptr;
}

bool match_element(const meta_element& me, const message& msg,
                   size_t pos, void** storage) {
  CAF_ASSERT(me.typenr != 0 || me.type != nullptr);
  if (!msg.match_element(pos, me.typenr, me.type)) {
    return false;
  }
  *storage = const_cast<void*>(msg.at(pos));
  return true;
}

bool match_atom_constant(const meta_element& me, const message& msg,
                         size_t pos, void** storage) {
  CAF_ASSERT(me.typenr == detail::type_nr<atom_value>::value);
  if (!msg.match_element(pos, detail::type_nr<atom_value>::value, nullptr)) {
    return false;
  }
  auto ptr = msg.at(pos);
  if (me.v != *reinterpret_cast<const atom_value*>(ptr)) {
    return false;
  }
  // This assignment casts `atom_value` to `atom_constant<V>*`.
  // This type violation could theoretically cause undefined behavior.
  // However, `uti` does have an address that is guaranteed to be valid
  // throughout the runtime of the program and the atom constant
  // does not have any members. Hence, this is nonetheless safe since
  // we are never actually going to dereference the pointer.
  *storage = const_cast<void*>(ptr);
  return true;
}

class set_commit_rollback {
 public:
  using pointer = void**;
  set_commit_rollback(const set_commit_rollback&) = delete;
  set_commit_rollback& operator=(const set_commit_rollback&) = delete;
  set_commit_rollback(pointer ptr) : m_data(ptr), m_pos(0), m_fallback_pos(0) {
    // nop
  }
  inline void inc() {
    ++m_pos;
  }
  inline pointer current() const {
    return &m_data[m_pos];
  }
  inline void commit() {
    m_fallback_pos = m_pos;
  }
  inline void rollback() {
    m_pos = m_fallback_pos;
  }
 private:
  pointer m_data;
  size_t m_pos;
  size_t m_fallback_pos;
};

bool try_match(const message& msg, size_t msg_pos, size_t msg_size,
               pattern_iterator pbegin, pattern_iterator pend,
               set_commit_rollback& storage) {
  while (msg_pos < msg_size) {
    if (pbegin == pend) {
      return false;
    }
    if (is_wildcard(*pbegin)) {
      // perform submatching
      ++pbegin;
      // always true at the end of the pattern
      if (pbegin == pend) {
        return true;
      }
      // safe current mapping as fallback
      storage.commit();
      // iterate over remaining values until we found a match
      for (; msg_pos < msg_size; ++msg_pos) {
        if (try_match(msg, msg_pos, msg_size, pbegin, pend, storage)) {
          return true;
        }
        // restore mapping to fallback (delete invalid mappings)
        storage.rollback();
      }
      return false; // no submatch found
    }
    // inspect current element
    if (!pbegin->fun(*pbegin, msg, msg_pos, storage.current())) {
      // type mismatch
      return false;
    }
    // next iteration
    storage.inc();
    ++msg_pos;
    ++pbegin;
  }
  // we found a match if we've inspected each element and consumed
  // the whole pattern (or the remainder consists of wildcards only)
  return std::all_of(pbegin, pend, is_wildcard);
}

bool try_match(const message& msg, pattern_iterator pb, size_t ps, void** out) {
  CAF_ASSERT(out != nullptr);
  CAF_ASSERT(msg.empty() || msg.vals()->get_reference_count() > 0);
  set_commit_rollback scr{out};
  return try_match(msg, 0, msg.size(), pb, pb + ps, scr);
}

} // namespace detail
} // namespace caf

==> actor-framework_0.16.3.orig.tar.gz: actor-framework-0.16.3/libcaf_core/src/try_match.cpp <==
/******************************************************************************
 *                       ____    _    _____                                   *
 *                      / ___|  / \  |  ___|    C++                           *
 *                     | |     / _ \ | |_       Actor                         *
 *                     | |___ / ___ \|  _|      Framework                     *
 *                      \____/_/   \_|_|                                      *
 *                                                                            *
 * Copyright 2011-2018 Dominik Charousset                                     *
 *                                                                            *
 * Distributed under the terms and conditions of the BSD 3-Clause License or  *
 * (at your option) under the terms and conditions of the Boost Software      *
 * License 1.0. See accompanying files LICENSE and LICENSE_ALTERNATIVE.       *
 *                                                                            *
 * If you did not receive a copy of the license files, see                    *
 * http://opensource.org/licenses/BSD-3-Clause and                            *
 * http://www.boost.org/LICENSE_1_0.txt.                                      *
 ******************************************************************************/

#include "caf/detail/try_match.hpp"

#include "caf/type_erased_tuple.hpp"

namespace caf {
namespace detail {

using pattern_iterator = const meta_element*;

bool match_element(const meta_element& me, const type_erased_tuple& xs,
                   size_t pos) {
  CAF_ASSERT(me.typenr != 0 || me.type != nullptr);
  return xs.matches(pos, me.typenr, me.type);
}

bool match_atom_constant(const meta_element& me, const type_erased_tuple& xs,
                         size_t pos) {
  CAF_ASSERT(me.typenr == type_nr<atom_value>::value);
  if (!xs.matches(pos, type_nr<atom_value>::value, nullptr))
    return false;
  auto ptr = xs.get(pos);
  return me.v == *reinterpret_cast<const atom_value*>(ptr);
}

bool try_match(const type_erased_tuple& xs,
               pattern_iterator iter, size_t ps) {
  if (xs.size() != ps)
    return false;
  for (size_t i = 0; i < ps; ++i, ++iter)
    // inspect current element
    if (!iter->fun(*iter, xs, i))
      // type mismatch
      return false;
  return true;
}

} // namespace detail
} // namespace caf
